/*
SAVList (C) 2005 Tatsuhiko Shoji
The sources for SAVList are distributed under the MIT open source license
*/
#include	<stdio.h>
#include	<stdlib.h>
#include	<ctype.h>

#include	<windows.h>
#include	<commctrl.h>

#include	"savFile.h"
#include	"fatFs.h"
#include	"readSav.h"

static FILE *fpSav = NULL;
int *entryOnListView = NULL;
int listViewEntrys;
int directoryEntrys;	/* fBNgGg */
int isSubDirectory = 0;
unsigned long fs_startCluster;

/* ztbs[t@Cɏ߂B */
int flushSavfile(void)
{
	int i,j;
	unsigned long now_cluster;
	unsigned long clusterSize;
	
	/* FAT */
	for (i = 0;i < 2;i++){
		for (j = 0;j < 3;j++){
			writeSector(fpSav,1 + (i * 3) + j,fat + SECTORSIZE * j );
		}
	}

	if (!isSubDirectory){
		/* [gfBNg */
		for (i = 0;i < 7;i++){
			writeSector(fpSav,7 + i,DirEntryBuf + (SECTORSIZE * i));
		}
	}else{
		/* FATǂăTufBNgށB */
		clusterSize = SECTORSIZE * SectorPerCluster;
		now_cluster = fs_startCluster;
		i = 0;
		do{
			writeCluster(fpSav,now_cluster,DirEntryBuf + clusterSize * i);
			i++;
			now_cluster = getFat12(now_cluster);
		}while(now_cluster < 0x0ff7l);
	}

	return 0;
}

/* ztbs[t@CJďǂݍށB */
int openSavfile(char *savFile)
{
	int i,result;
	
	fpSav = fopen(savFile,"r+b");
	if (fpSav == NULL){
		fprintf(stderr,"Can't open virtual floppy disk file.\n");
		return 1;
	}
	fprintf(stderr,"Initializing.\n");
	result = initialize(fpSav);
	if (result){
		fclose(fpSav);
		return 1;
	}

	fprintf(stderr,"Memory allocating.\n");
	fat = calloc(3,SECTORSIZE);
	if (fat == NULL){
		fprintf(stderr,"No enough memory.\n");
		fclose(fpSav);
		return 1;
	}
	DirEntryBuf = calloc(7,SECTORSIZE);
	if (DirEntryBuf == NULL){
		fprintf(stderr,"No enough memory.\n");
		free(fat);
		fclose(fpSav);
		return 1;
	}

	fprintf(stderr,"Reading disk information.\n");
	/* FATǂݍ */
	for (i = 0;i < 3;i++){
		readSector(fpSav,1 + i,fat + (SECTORSIZE * i));
	}
	setFat12(0,0xff9);
	setFat12(1,0xfff);
	

	/* [gfBNgǂݍ */
	for (i = 0;i < 7;i++){
		readSector(fpSav,7 + i,DirEntryBuf + (SECTORSIZE * i));
	}
	isSubDirectory = 0;

	return 0;
}

/* \[XJ */
void cleanup(void)
{
	if (DirEntryBuf != NULL){
		free(DirEntryBuf);
		DirEntryBuf = NULL;
	}
	if (fat != NULL){
		free(fat);
		fat = NULL;
	}
	if (fpSav != NULL){
		fclose(fpSav);
		fpSav = NULL;
	}
	if (entryOnListView != NULL){
		free(entryOnListView);
		entryOnListView = NULL;
	}
}

/* w肳ꂽt@C̏fBNgɃZbgB */
/* WindowsɈˑB */
int setupDirectory(struct dirEntry *d,char *filename)
{
	HANDLE hFind;
	WIN32_FIND_DATA info;
	unsigned short time,date;
	int i;
	char name[_MAX_FNAME],ext[_MAX_EXT];

	/* t@C̏𓾂B */
	hFind = FindFirstFile(filename,&info);
	if (hFind == INVALID_HANDLE_VALUE){
		fprintf(stderr,"File not found.\n");
		return 2;
	}
	FindClose(hFind);

	/* fBXN̋󂫂t@CTCY̕傫G[ƂĔB */
	if (info.nFileSizeHigh > 0){
		/* fprintf(stderr,"No disk space.\n"); */
		return 1;
	}

#if 0
	cl = getFreeClusterCount();
	imgSize = SECTORSIZE * SectorPerCluster * cl;
	if (imgSize < info.nFileSizeLow){
		/* fprintf(stderr,"No disk space.\n"); */
		return 1;
	}
#endif

	memset(d,0x00,sizeof(struct dirEntry));

	/* t@C̏fBNgɐݒ肵ĂB */
	/* ^CX^v */
	FileTimeToDosDateTime(&(info.ftLastWriteTime),&date,&time);

	d->time = time;
	d->date = date;

	/* t@CTCY */
	d->size = info.nFileSizeLow;
	for (i = 0;i < 8;i++){
		d->filename[i] = ' ';
	}
	for (i = 0;i < 3;i++){
		d->ext[i] = ' ';
	}

	/* t@C */
	_splitpath(filename,NULL,NULL,name,ext);
	/* printf("%s %s\n",name,ext); */
	for (i = 0;i < 8;i++){
		if (name[i] != 0x00){
			d->filename[i] = toupper(name[i]);
		}else{
			break;
		}
	}
	for (i = 0;i < 3;i++){
		if (ext[i+1] != 0x00){
			d->ext[i] = toupper(ext[i+1]);
		}else{
			break;
		}
	}
	return 0;
}

/* TufBNgɃfBNgGgǉB */
int addDirectoryEntry(unsigned long startCluster)
{
	int i;
	unsigned long now_cluster,next_cluster;
	unsigned long newCluster;	/* tNX^ */
	unsigned long clusterSize;
	unsigned long clusterCount;
	unsigned char *newDirectory;
	int *newEntryOnListView;

	newCluster = getFreeCluster();
	if (newCluster == 0){
		/* tGgȂ */
		return 1;
	}

	/* FATǂăTufBNgށB */
	clusterSize = SECTORSIZE * SectorPerCluster;
	/* fBNgGg̍Ō̃NX^oB */
	now_cluster = startCluster;
	next_cluster = startCluster;
	clusterCount = 0;
	i = 0;
	do{
		now_cluster = next_cluster;
		/* writeCluster(fpSav,now_cluster,DirEntryBuf + clusterSize * i); */
		i++;
		clusterCount++;
		next_cluster = getFat12(now_cluster);
	}while(next_cluster < 0x0ff7l);
	/* FATȂB */
	setFat12(now_cluster,newCluster);
	setFat12(newCluster,0xfffl);

	/* g債TCỸGg̃mۂB */
	newDirectory = calloc(clusterSize * (clusterCount + 1),1);
	if (newDirectory == NULL){
		return 1;
	}
	/* fBNgGgV̂ɕtւB */
	memcpy(newDirectory,DirEntryBuf,clusterSize * clusterCount);
	free(DirEntryBuf);
	DirEntryBuf = newDirectory;

	/* fBNgɓGgXVB */
	directoryEntrys = (clusterSize * (clusterCount + 1)) / 32;

	/* fBNgGgēxmۂ */
	newEntryOnListView = (int *)malloc(sizeof(int) * directoryEntrys);
	if (newEntryOnListView == NULL){
		return 1;
	}
	entryOnListView = newEntryOnListView;

	return 0;
}

/* fBNgGgɋ󂫂邩ǂ`FbNB */
int checkEmptyEntry(struct dirEntry *d)
{
	int i,empty;
	struct dirEntry d2;
	unsigned long countFree;
	unsigned long imgSize;

	/* ztbs[fBXNt@CɃfBNgGg̋󂫂邩H */
	empty = -1;
	for (i = 0;i < directoryEntrys;i++){
		getDirectory(&d2,i);
		if ((d2.filename[0] == 0x00) || (d2.filename[0] == (char)0xe5)){
			if (empty == -1){
				/* ܂A̋󂫂ĂȂ炱̏ꏊɂB */
				empty = i;
				/* ŏIfBNgGgȊOŃGg̃Gh}[N */
				/* ̃GgɃGh}[NB */
				if ((d2.filename[0] == 0x00) && (i < (directoryEntrys - 1))){
					setDirectory(&d2,i+1);
				}
			}
			/* Gh}[N炱̐̃fBNgGg͖Ȃ̂ŔB */
			if (d2.filename[0] == 0x00){
				break;
			}
		}
		/* w肳ꂽt@Cłɑ݂Slď݂sȂ悤ɂB */
		if (memcmp(d->filename,d2.filename,8) == 0){
			if (memcmp(d->ext,d2.ext,3) == 0){
#ifdef NOOVERWRITE				
				fprintf(stderr,"File already exists.\n");
				return -1;
#else
				/* 󂭃NX^vZċ󂫂邩ǂ𔻒肷B */
				countFree = countClusters(d2.cluster);
				countFree += getFreeClusterCount();
				imgSize = SECTORSIZE * SectorPerCluster * countFree;
				if (imgSize < d->size){
					/* t@CTCY̕傫ꍇ͔B */
					return -2;
				}

				/* ̃GggpB */
				clearFatChain((unsigned long)(d2.cluster & 0x0fff));
				empty = i;
#endif
			}
		}
	}
	return empty;
}

/* t@C݂̏sB */
int writeFile(char *filename)
{
	struct dirEntry d;
	int result,empty;
	size_t read;
	unsigned long cl,next;
	char buf[1024];
	FILE *fp;
	unsigned long countFree;
	unsigned long imgSize;

	/* fBNgZbgĂȂꍇAȂB */
	if (DirEntryBuf == NULL)
		return 1;
	if (entryOnListView == NULL)
		return 1;

	result = setupDirectory(&d,filename);
	if (result > 0){
		return 1;
	}

	/* 󂫃Gg邩ǂׂ */
	empty = checkEmptyEntry(&d);

	if (empty == -2){
		/* łɋ󂫗eʂȂꍇ͔B */
		return NO_DISKSPACE;
	}
	if ((empty == -1) && (isSubDirectory)){
		/* TufBNg̏ꍇAGggłȂǂmFB */
		result = addDirectoryEntry(fs_startCluster);
		if (result){
			/* Ggg傷邱Ƃ͂łȂ */
			return NO_DISKSPACE;
		}
		/* g債Gg󂫂TB */
		empty = checkEmptyEntry(&d);
		if (empty < 0){
			return NO_ENTRY;
		}
	}

	/* 󂫃GgȂꍇ͔ */
	if (empty < 0){
		return NO_ENTRY;
	}

	/* 󂫃NX^vZċ󂫂邩ǂ𔻒肷B */
	countFree = getFreeClusterCount();
	imgSize = SECTORSIZE * SectorPerCluster * countFree;
	if (imgSize < d.size){
		/* t@CTCY̕傫ꍇ͔B */
		return NO_DISKSPACE;
	}

	/* fBXNɏ */
	cl = getFreeCluster();	/* ŏ̋󂫃NX^ԍ */
	d.cluster = (unsigned short)cl;	/* fBNgGg̊JnNX^ */

	fp = fopen(filename,"rb");
	if (fp == NULL){
		return 1;
	}

	/* ŏ1߂̃NX^ */
	read = fread(buf,1,1024,fp);
	if (read != 0){
		/* ŏ1NX^ڂ̏ */
		if (cl == 0){
			return NO_DISKSPACE;
		}

		writeCluster(fpSav,cl,buf);
		setFat12(cl,0xfff);	/* ꉞAǂݍ񂾎_FAT𓯊Ă */
		next = getFreeCluster();
		if (next == 0){
			return NO_DISKSPACE;
		}

		while(1){
			read = fread(buf,1,1024,fp);
			if (read == 0){
				/* xNX^PʂɎ܂ƂFATqȂB */
				break;
			}else{
				/* FATGgqB */
				setFat12(cl,next);
			}
			/* printf("."); */
			/* ǂݍ񂾓eNX^ݒ肷B */
			cl = next;
			writeCluster(fpSav,cl,buf);
			setFat12(cl,0xfff);	/* ꉞAǂݍ񂾎_FAT𓯊Ă */
			if (read == 1024){
				/* ̃NX^ւȂB */
				next = getFreeCluster();
				/* printf("Now:%lx Next:%lx\n",cl,next); */
			}else{
				/* t@Cǂ݂ꍇ͂ŔB */
				setFat12(cl,0xfff);
				break;
			}
		}
	}else{
		/* 0oCg̃t@C̏ꍇANX^Gg0ɂȂB */
		d.cluster = (unsigned short)0;	/* fBNgGg̊JnNX^ */
	}

	fclose(fp);
	/* 󂫃GgɃfBNg̏ݒ肷B */
	setDirectory(&d,empty);

	/* t@C݂ɐFATƃ[gfBNgށB */
	flushSavfile();

	return 0;
}

/* t@CWJ */
int extractFile(char *dir,int listviewEntry)
{
	int entry,i;
	size_t len;
	char buf[_MAX_PATH];
	char *clusterBuf;
	struct dirEntry d;
	unsigned long size;
	unsigned long clusterSize;
	unsigned long clusterNo;
	FILE *fpo;

	/* fBNgZbgĂȂꍇAȂB */
	if (DirEntryBuf == NULL)
		return 1;
	if (entryOnListView == NULL)
		return 1;

	/* fBNgGg擾 */
	entry = entryOnListView[listviewEntry];
	getDirectory(&d,entry);

	/* pX */
	strcpy(buf,dir);
	len = strlen(dir);
	if (dir[len - 1] != '\\'){
		strcat(buf,"\\");
		len++;
	}
	for (i = 0;i < 8;i++){
		if (d.filename[i] == ' '){
			break;
		}
		buf[len] = d.filename[i]; 
		len++;
	}
	if (d.ext[0] != ' '){
		buf[len] = '.'; 
		len++;
		for (i = 0;i < 3;i++){
			if (d.ext[i] == ' '){
				break;
			}
			buf[len] = d.ext[i]; 
			len++;
		}
	}
	buf[len] = '\0';
	len++;

	size = d.size;
	clusterSize = SECTORSIZE * SectorPerCluster;
	clusterNo = d.cluster;
	clusterBuf = malloc((size_t)clusterSize);
	if (clusterBuf == NULL){
		return 1;
	}

	fpo = fopen(buf,"wb");
	if (fpo == NULL){
		free(clusterBuf);
		return 1;
	}
	while(size >= clusterSize){
		readCluster(fpSav,clusterNo,clusterBuf);
		fwrite(clusterBuf,1,clusterSize,fpo);

		clusterNo = getFat12(clusterNo);
		size = size - clusterSize;
		if (clusterNo >= 0x00000ff7){
			break;
		}
	}
	if (size > 0){
		readCluster(fpSav,clusterNo,clusterBuf);
		fwrite(clusterBuf,1,size,fpo);
	}
	fclose(fpo);
	free(clusterBuf);

	return 0;
}

/* fBNgǂݍ */
void readDirectory(unsigned long startCluster)
{
	unsigned long i;
	unsigned long clusterCount;
	unsigned long clusterPos;
	unsigned long nextCluster;
	unsigned long clusterSize;

	clusterSize = SECTORSIZE * SectorPerCluster;

	if (DirEntryBuf != NULL){
		free(DirEntryBuf);
		DirEntryBuf = NULL;
	}
	if (entryOnListView != NULL){
		free(entryOnListView);
		entryOnListView = NULL;
	}

	if (startCluster == 0){
		directoryEntrys = SECTORSIZE * 7 / 32;	/* fBNgGg */

		DirEntryBuf = malloc((size_t)(SECTORSIZE * 7));	/* fBNg̕ēxmۂB */
		if (DirEntryBuf == NULL){
			return;
		}

		/* fBNgGgēxmۂ */
		entryOnListView = (int *)malloc(sizeof(int) * directoryEntrys);
		if (entryOnListView == NULL){
			return;
		}

		/* [gfBNgǂݍ */
		for (i = 0;i < 7;i++){
			readSector(fpSav,7 + i,DirEntryBuf + (SECTORSIZE * i));
		}
		isSubDirectory = 0;
	}else{
		clusterCount = countClusters(startCluster);	/* fBNgɊ蓖ĂꂽNX^ */
		directoryEntrys = clusterCount * clusterSize / 32;	/* fBNgGg */

		DirEntryBuf = malloc((size_t)(clusterSize * clusterCount));	/* fBNg̕ēxmۂB */
		if (DirEntryBuf == NULL){
			return;
		}

		/* fBNgGgēxmۂ */
		entryOnListView = (int *)malloc(sizeof(int) * directoryEntrys);
		if (entryOnListView == NULL){
			return;
		}

		nextCluster = startCluster;
		clusterPos = 0;
		do {
			readCluster(fpSav,nextCluster,DirEntryBuf + clusterSize * clusterPos);
			clusterPos++;
			nextCluster = getFat12(nextCluster);
		}while(nextCluster < 0x0ff7l);
		isSubDirectory = 1;
	}
}

void getSavStatus(struct SavStatus *status)
{
	int i;
	unsigned long clusterSize;
	unsigned long files = 0,directorys = 0;
	struct dirEntry d2;

	/* 󂫗eʌvZ */
	clusterSize = SECTORSIZE * SectorPerCluster;
	status->freeSize = getFreeClusterCount() * clusterSize;
	status->maxSize = MaxEntry * clusterSize;


	/* ztbs[fBXNt@CɃfBNgGg̋󂫂邩H */
	for (i = 0;i < directoryEntrys;i++){
		getDirectory(&d2,i);
		if (d2.filename[0] == 0x00){
			break;
		}
		if ((unsigned char)(d2.filename[0]) == 0xe5){
			continue;
		}
		if (d2.attr & 0x10){
			directorys++;
		}else{
			if (d2.attr & 0x08){
				/* {[xOt@C */
			}else{
				files++;
			}
		}
	}

	status->directorys = directorys;
	status->files = files;
}

/* SAVt@Cǂݍ܂ */
int readSavFile(char *filename)
{
	int result;
	
	/* fBXN̏ݒ肷B */
	FatSectors = 3;	/* FATp̃ZN^ */
	MaxEntry = (SECTORS - 14) / 2 + 2;	/* őFATGg */
	SectorPerCluster = 2;	/* 1NX^\ZN^ */
	DataArea = 14;	/* f[^GAJnZN^ */
	directoryEntrys = 112;	/* ݂̃fBNg̃Gg */
	isSubDirectory = 0;	/* [gfBNg */

	result = openSavfile(filename);
	if (result > 0){
		return 2;
	}

	entryOnListView = (int *)malloc(sizeof(int) * directoryEntrys);
	if (entryOnListView == NULL){
		return 2;
	}

	return 0;
}

/*-----------------------*/
/* ̐GUIAgp֐ */
/*-----------------------*/

/* fBNgĕ\ */
void refreshDir(HWND hList)
{
	int i,pos;
	LV_ITEM item;
	char buf[32];
	struct dirEntry d;
	int listCount = 0;	/* XgɓĂGg */

	if (DirEntryBuf == NULL)
		return;
	if (entryOnListView == NULL)
		return;

	memset(&item,0,sizeof(LV_ITEM));
	ListView_DeleteAllItems(hList);

	pos = 0;
	for (i = 0;i < directoryEntrys;i++){
		item.mask = LVIF_TEXT;

		/* fBNgGg擾B */
		getDirectory(&d,i);

		/* fBNgGg̏Ȉꍇ\ł؂B */
		if (d.filename[0] == 0x00){
			break;
		}

		/* 폜ꂽGg͕\ȂB */
		if ((unsigned char)(d.filename[0]) == 0xe5){
			continue;
		}

		/* Ot@C͕\Ȃ */
		if (d.attr == 0x0f){
			continue;
		}

		/* fBNgGg̈ʒuƃXgr[̈ʒu֘At */
		entryOnListView[listCount] = i;
		listCount++;

		/* fBNgGg̈ʒu */
		sprintf(buf,"%d",i);
		item.pszText = buf;
		item.iItem = pos;
		item.iSubItem = 0;
		ListView_InsertItem(hList,&item);

		/* t@C */
		memset(buf,0x00,32);
		memcpy(buf,d.filename,8);
		if (d.ext[0] != ' '){
			buf[8] = '.';
			memcpy(buf + 9,d.ext,3);
		}

		item.pszText = buf;
		item.iItem = pos;
		item.iSubItem = 1;
		ListView_SetItem(hList,&item);

		/* t@CTCY */
		if (d.attr & 0x10){
			sprintf(buf,"<DIR>");
		}else if (d.attr & 0x08){
			sprintf(buf,"<VOL>");
		}else{
			memset(buf,0x00,32);
			sprintf(buf,"%lu",d.size);
		}

		item.pszText = buf;
		item.iItem = pos;
		item.iSubItem = 2;
		ListView_SetItem(hList,&item);

		/* XV */
		sprintf(buf,"%04d/%02d/%02d %02d:%02d:%02d",
			((d.date >> 9) & 0x7f) + 1980,
			(d.date >> 5) & 0x0f,
			(d.date) & 0x1f,
			(d.time >> 11) & 0x1f,
			(d.time >> 5) & 0x3f,
			(d.time & 0x1f) << 1);

		item.pszText = buf;
		item.iItem = pos;
		item.iSubItem = 3;
		ListView_SetItem(hList,&item);

		pos++;
	}

	listViewEntrys = listCount;
}

/* t@CׂđI */
void selectAllFiles(HWND hList)
{
	int i,pos;
	struct dirEntry d;
	int listCount = 0;	/* XgɓĂGg */

	pos = 0;
	for (i = 0;i < directoryEntrys;i++){
		/* fBNgGg擾B */
		getDirectory(&d,i);

		/* fBNgGg̏Ȉꍇ\ł؂B */
		if (d.filename[0] == 0x00){
			break;
		}

		/* 폜ꂽGg͕\ȂB */
		if ((unsigned char)(d.filename[0]) == 0xe5){
			continue;
		}

		/* Ot@C͕\Ȃ */
		if (d.attr == 0x0f){
			continue;
		}

		if (d.attr & 0x10){
			/* fBNg */
			ListView_SetItemState(hList,listCount,0,LVIS_SELECTED);
		}else if (d.attr & 0x08) {
			/* {[x */
			ListView_SetItemState(hList,listCount,0,LVIS_SELECTED);
		}else{
			ListView_SetItemState(hList,listCount,LVIS_SELECTED,LVIS_SELECTED);
		}

		/* fBNgGg̈ʒuƃXgr[̈ʒu֘At */
		listCount++;
	}

}

/* t@CׂđI */
int upDirectory(HWND hList)
{
	int i,pos;
	struct dirEntry d;
	int listCount = 0;	/* XgɓĂGg */

	pos = 0;
	for (i = 0;i < directoryEntrys;i++){
		/* fBNgGg擾B */
		getDirectory(&d,i);

		/* fBNgGg̏Ȉꍇł؂B */
		if (d.filename[0] == 0x00){
			break;
		}

		/* 폜ꂽGg͔΂B */
		if ((unsigned char)(d.filename[0]) == 0xe5){
			continue;
		}

		/* Ot@C͔΂ */
		if (d.attr == 0x0f){
			continue;
		}

		if (d.attr & 0x10){
			/* fBNg */
			if (d.filename[0] == '.' &&
				d.filename[1] == '.' &&
				d.filename[2] == ' '){
				enterDirectory(hList,i);
				return 1;
			}
		}

		/* fBNgGg̈ʒuƃXgr[̈ʒu֘At */
		listCount++;
	}
	return 0;
}

/* TufBNg̒ɓ */
void enterDirectory(HWND hList,int listviewEntry)
{
	int entry;
	struct dirEntry d;

	if (DirEntryBuf == NULL)
		return;
	if (entryOnListView == NULL)
		return;

	/* fBNgGg擾 */
	entry = entryOnListView[listviewEntry];
	getDirectory(&d,entry);

	if (d.attr & 0x10){
		if (d.filename[0] == '.' && d.filename[1] == ' '){
			/* .(̃fBNg)̏ꍇȂ */
		}else{
			/* fBNgǂݍ */
			readDirectory(d.cluster);
			fs_startCluster = d.cluster;
			/* fBNg̓eXgr[ɓ] */
			refreshDir(hList);
		}
	}

}

/* It@C폜 */
void deleteSelectedFiles(HWND hList)
{
	int i,entry;
	struct dirEntry d;
	UINT state;

	for (i = 0;i < listViewEntrys;i++){
		state = ListView_GetItemState(hList,i,LVIS_SELECTED);
		if (!state){
			continue;
		}
		entry = entryOnListView[i];

		/* fBNgGg擾B */
		getDirectory(&d,entry);

		if (d.attr & 0x10){
			/* fBNg */
			continue;
		}else if (d.attr & 0x08) {
			/* {[x */
			continue;
		}else{
			d.filename[0] = 0xe5;
			if (d.cluster != 0){
				clearFatChain(d.cluster);
			}
			setDirectory(&d,entry);
		}

	}

	/* t@C폜ɐFATƃfBNgށB */
	flushSavfile();
}

/* TufBNg̒ɓ */
void deleteLongFilename(HWND hList)
{
	int i,pos;
	struct dirEntry d;
	int listCount = 0;	/* XgɓĂGg */

	pos = 0;
	for (i = 0;i < directoryEntrys;i++){
		/* fBNgGg擾B */
		getDirectory(&d,i);

		/* fBNgGg̏Ȉꍇ\ł؂B */
		if (d.filename[0] == 0x00){
			break;
		}

		/* 폜ꂽGg͕\ȂB */
		if ((unsigned char)(d.filename[0]) == 0xe5){
			continue;
		}

		if ((d.attr & 0x0f) == 0x0f){
			d.filename[0] = 0xe5;
			setDirectory(&d,i);
		}
	}
	/* t@C폜ɐFATƃfBNgށB */
	flushSavfile();

}

